"""
forms for deviations app
"""
from analyses.deviations import models as app_models
from installations import models as installations_models
from django import forms
from utils import forms as utils_forms
from utils import fields as utils_fields
from utils import lists as utils_lists
from utils import widgets as utils_widgets
from utils import sheets
from utils.versioning import versionedforms
from utils.versioning.models import WorkVersion
from django.utils.translation import ugettext_lazy as _

class DeviationScenarioLOPAForm(versionedforms.VersionedForm):
    class Meta:
        model = app_models.DeviationScenario
        fields = ('use_LOPA', 'target_frequency', 'motivation')
        attrs = {
            'motivation': {'rows':10,'cols':65},
        }
        widgets = {
            'motivation': utils_widgets.MarkItUpWidget(),
        }

class DeviationScenarioLOPASheet(versionedforms.VersionedSheet):
    final = sheets.SheetField(data='final', name='final', label=_('Final event'))
    class Meta:
        model = app_models.DeviationScenario
        fields = ('use_LOPA', 'target_frequency', 'motivation', 'final')
    def __init__(self, instance, *args, **kwargs):
        super(DeviationScenarioLOPASheet, self).__init__(instance, *args, **kwargs)
        if not instance.use_LOPA:
            del self.fields['target_frequency']
            del self.fields['motivation']
            del self.fields['final']

class DeviationScenarioCompareSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
        model = app_models.DeviationScenario
        fields = ('name', 'type', 'description', 'evaluation', 'use_LOPA', 'target_frequency', 'motivation', 'is_ok', 'scope', 'suggestionlist')
    def __init__(self, instance, *args, **kwargs):
        super(DeviationScenarioCompareSheet, self).__init__(instance, *args, **kwargs)
        if instance.suggestionlist:
            del self.fields['scope']
        else:
            del self.fields['suggestionlist']

class DeviationScenarioReportSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    evaluation = utils_fields.MarkdownField(data = 'evaluation', name='evaluation', label=_('Evaluation'))
    class Meta:
        model = app_models.DeviationScenario
        fields = ('type', 'description', 'evaluation', 'target_frequency', 'motivation', 'scope', 'suggestionlist')
    def __init__(self, instance, *args, **kwargs):
        super(DeviationScenarioReportSheet, self).__init__(instance, *args, **kwargs)
        if instance.suggestionlist:
            del self.fields['scope']
        else:
            del self.fields['suggestionlist']
        if not instance.use_LOPA:
            del self.fields['target_frequency']


class DependencyForm(versionedforms.VersionedForm):
    class Meta:
        model = app_models.Dependency
        fields = ('element1', 'element2', 'description')
        attrs = {
            'description': {'rows':10,'cols':65},
        }

    def __init__(self, version=None, scenario=None, *args, **kwargs):
        super(DependencyForm, self).__init__(*args, **kwargs)
        self.fields['element1'].label = _('Protection layer')
        self.fields['element1'].queryset = app_models.CauseTreeElement.objects.filter(
            scenario=scenario,
            elementtype=app_models.PROTECTIONLAYER_ID
            ).get_versionedobjects(version=version)
        self.fields['element2'].queryset = app_models.CauseTreeElement.objects.filter(
            scenario=scenario,
            elementtype__in = (app_models.PROTECTIONLAYER_ID, app_models.EVENT_ID)
        ).get_versionedobjects(version=version)

    def clean(self):
        cleaned_data = self.cleaned_data
        if cleaned_data.get('element1') == cleaned_data.get('element2'):
            raise forms.ValidationError(_('You cannot set a dependency of an element to itself!'))
        return cleaned_data

class DependencySheet(versionedforms.VersionedSheet):
    class Meta:
        model = app_models.Dependency

class DependencyReportSheet(versionedforms.VersionedSheet):
    class Meta:
        model = app_models.Dependency

class ProtectionLayerForm(versionedforms.VersionedForm):
    class Meta:
        model = app_models.ProtectionLayer
        me_model = app_models.DeviationMeasure
        attrs = {
            'name': {'size':75},
            'description': {'rows':10,'cols':65},
            'effectivity': {'rows':5,'cols':65},
            'reliability': {'rows':5,'cols':65},
            'risks': {'rows':5,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget(),
            'effectivity': utils_widgets.MarkItUpWidget(),
            'reliability': utils_widgets.MarkItUpWidget(),
            'risks': utils_widgets.MarkItUpWidget(),
        }
        exclude = ('SL',)

    def __init__(self, *args, **kwargs):
        super(ProtectionLayerForm, self).__init__(*args, **kwargs)
        instance = kwargs.get('instance')
        if instance:
            el =  instance.get_treeelements()
            if el:
                if not el[0].scenario.use_LOPA:
                    del self.fields['reltype']
                    del self.fields['rel']


class RelSheetField(sheets.TextField):
    def get_label(self, instance):
        return u'%s:' % instance.get_reltype_display()

# Add pl sheet: change caption of rel
class ProtectionLayerSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    effectivity = utils_fields.MarkdownField(data = 'effectivity', name='effectivity', label=_('Effectivity'))
    reliability = utils_fields.MarkdownField(data = 'reliability', name='reliability', label=_('Reliability'))
    risks = utils_fields.MarkdownField(data = 'risks', name='risks', label=_('Risks'))
    rel = RelSheetField(data='rel', name='rel', label=_('PFD / FF'))
    class Meta:
        model = app_models.ProtectionLayer
        exclude = ('reltype', 'SL',)
    def __init__(self, instance, report=False, *args, **kwargs):
        super(ProtectionLayerSheet, self).__init__(instance, report=report, *args, **kwargs)
        if instance.get_treeelements() and instance.get_treeelements()[0].scenario.use_LOPA:
            # this should be in the RelSHeetField
            if instance.reltype == 3:
                self.fields['rel'].data = lambda x: x.rel and u'%s %%' % (x.rel * 100) or u''
        else:
            del self.fields['rel']
        if report:
            del self.fields['name']

class CauseForm(versionedforms.VersionedForm):
    class Meta:
        model = app_models.Cause
        fields = ('name', 'description', 'condition', 'frequency', 'probability')
        attrs = {
            'name': {'size':75},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }

    def __init__(self, *args, **kwargs):
        super(CauseForm, self).__init__(*args, **kwargs)
        instance = kwargs.get('instance')
        if instance:
            el =  instance.get_treeelements()
            if el:
                if not el[0].scenario.use_LOPA:
                    del self.fields['frequency']
                    del self.fields['probability']


class CauseSheet(versionedforms.VersionedSheet):
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
        model = app_models.Cause
    def __init__(self, instance, report=False, *args, **kwargs):
        super(CauseSheet, self).__init__(instance, report=report, *args, **kwargs)
        if not instance.get_treeelements()[0].scenario.use_LOPA:
            del self.fields['frequency']
            del self.fields['probability']
        else:
            if instance.condition:
                del self.fields['frequency']
            else:
                del self.fields['probability']
        if report:
            del self.fields['name']


class LopaPathFilterForm(utils_lists.FilterForm):
    """Custom filterform for the lopapath list"""
    high = forms.NullBooleanField(required=False, label=_('Freq. high'),)

