# task views

import datetime

from django.core.urlresolvers import reverse
from django.db.models.loading import get_model
from django.db.models import Count, F, Q
from django.utils.translation import ugettext_lazy as _

from installations.models import SectionTreeElement

from tasks import models as app_models
from tasks import forms as app_forms
from tasks import tables as app_tables
from tasks.filter import get_tasks_for_installation

from utils import crud
from utils import forms as utils_forms
from utils import permissions
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version

class MixIn(object):
    "Common items for object views"
    model = app_models.Task
    include_attachments = True
    include_links = True

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  [
        {'name': 'tasks',
            'url': reverse('tasks'),
            'label': _('Tasks')},
        ]

class display(MixIn, versionedcrud.DisplayObjectView):
    form_class = app_forms.TaskSheet
    include_tasks = False
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.TaskForm

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.TaskSheet

    def get_default_url(self, object):
        if object.content_object:
            return display_url_for_obj(object.content_object)
        return reverse('tasks')

class report(MixIn, versionedcrud.ReportObjectView):
    pass

class add(versionedcrud.AddObjectView):
    """
    Add a task to an object.
    """
    model = app_models.Task
    form_class = app_forms.TaskForm
    parent_field = 'content_object'

    def __call__(self, request, app_label, model_name, *args, **kwargs):
        self.parent_class = get_model(app_label, model_name)
        return super(add, self).__call__(request, *args, **kwargs)

class display_for(versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    template = 'tasks/display.html'
    allow_report = False
    def __call__(self, request, app_label, model_name, *args, **kwargs):
        self.model = get_model(app_label, model_name)
        return super(display_for, self).__call__(request, *args, **kwargs)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        queryset = app_models.Task.objects.tasks_for_object(object)
        table = app_tables.TasksTable(queryset, skipcolumns=['content_object',])
        buttons = []
        if self.request.user.has_perm('tasks.edit_tasks'):
            buttons.append(
                {'url': add_url_for_obj(object),
                    'label': _('Add')},)
        table.buttons = buttons
        extra_context['table'] = table
        extra_context['breadcrumbs'] = [{'url': object.get_absolute_url(), 'label': u'%s' % object},]

def add_url_for_obj(obj):
    return reverse('tasks_add', kwargs={
                        'app_label': obj._meta.app_label,
                        'model_name': obj._meta.model_name,
                        'object_id': obj.v_id
                    })

def display_url_for_obj(obj):
    return reverse('tasks_display_for', kwargs={
                        'app_label': obj._meta.app_label,
                        'model_name': obj._meta.model_name,
                        'object_id': obj.v_id
                    })


class TaskFilter(crud.Filter):
    form = app_forms.TaskFilterForm

    def __call__(self, request, queryset, **kwargs):
        """
        pass version into form
        """
        (version, specialversion) = get_current_version(request)
        return super(TaskFilter, self).__call__(request, queryset, version=version)

    def task_completed_filter(item):
        """
        callback function to supply parameters for filter form
        must return a list of qobjects and a dictionary of params
        """
        Qobjs = []
        params = {}
        if item == u'1':
            # complete
            params['completiondate__isnull'] = False
        elif item == u'2':
        # incomplete
            params['completiondate__isnull'] = True
        elif item == u'3':
        # incomplete, late
            params['completiondate__isnull'] = True
            params['duedate__lt'] = datetime.datetime.now().date()
        elif item == u'4':
        # completed on time
            params['completiondate__lte'] = F('duedate')
        elif item == u'5':
        # completed over time
            params['completiondate__gt'] = F('duedate')
        return Qobjs, params

    filters = [
                {'filteritem': 'task', 'parameter': 'task__icontains', 'type': 'normal'},
                {'filteritem': 'user', 'parameter': 'user__exact', 'type': 'normal'},
                {'filteritem': 'complete', 'function': task_completed_filter, 'type': 'function'},
            ]

    def apply_filter(self, user, queryset, Qobjs, params, cleaned_data):
        qs, filtered = self.extra_filter(queryset, cleaned_data)
        if cleaned_data.get('installation'):
            qs = get_tasks_for_installation(cleaned_data['installation'], Qobjs, params, user)
            filtered = True
        else:
            qs = qs.filter(*Qobjs, **params).distinct()
        return qs, filtered

class TaskListObject(versionedcrud.TableListObject):
    table_class = app_tables.TasksTable
    defaultsort = 'duedate'
    queryset = app_models.Task.objects.all().select_related_versioned()
    title = _('Tasks')
    table_classes = ['tasks']

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('tasksheets'):
            html_strings.append('<pdf:nextpage>')
            for item in queryset:
                html_strings.extend(item.get_report_body(user, language, choices, level=2))
                html_strings.append('<hr />')

    def add_css(self, css_files, queryset, user, language, choices):
        css_files.extend(app_models.Task.get_listreport_css(language, choices))
        if choices.get('tasksheets'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class tasks(versionedcrud.TableListView):
    "List of tasks"

    table_class = app_tables.TasksTable
    csv_table_class = app_tables.TasksCSVTable
    defaultsort = 'duedate'
    queryset = app_models.Task.objects.all().select_related_versioned()
    paginate_by = 100
    title = _('Tasks')
    currentmenuitem = 'tasks'
    filter = TaskFilter()
##    redirect_on_unique = True
    allow_pdf = True
    allow_csv = True
    pdfselectform_class = utils_forms.BareListReportForm
    pdfsendmail = True
    listobject = TaskListObject()

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=app_models.Task)

    def add_to_extra_context(self, extra_context):
        super(tasks, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  [
        {'name': 'tasks',
            'url': reverse('tasks'),
            'label': _('Tasks')},
        ]
