"""
Attachments app

! Depends on the versioning app

Objects for now need to be VersionedModel subclasses

"""
import os
from django.conf import settings
from django.db import models
from django.db.models import signals as django_signals
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from utils import reports as utils_reports
from utils.versioning import models as versioning_models
from django.core.files.storage import FileSystemStorage

fs = FileSystemStorage(location=settings.MEDIA_ROOT, base_url = settings.MEDIA_URL)

class AttachmentManager(versioning_models.VersionedModelManager):
    def attachments_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        version = getattr(obj, '_version', obj.version) or versioning_models.WorkVersion
        return self.filter(content_type__pk=object_type.id,
                           object_id=obj.v_id).get_versionedobjects(version=version)

class Attachment(versioning_models.VersionedModel):
    def attachment_upload(instance, filename):
        """
        Stores the attachment in a "per module/appname/primary key" folder
        Trim the names if they are too long.
        """
        upperpath = '%s_%s' % (instance.content_object._meta.app_label,
                       instance.content_object._meta.object_name.lower())
        if len(upperpath) > 50:
            upperpath = upperpath[:50]
        path = 'attachments/%s/%s' % (upperpath, instance.content_object.pk)
        fn = '%s/%s' % (path, filename)
        if len(fn) > 100:
            try:
                dot = filename.rindex('.')
                maxlen = 96 - len(path) - len(filename) + dot
                newfilename = '%s%s' % (filename[:maxlen], filename[dot:])
            except:
                maxlen = 96 - len(path)
                newfilename = filename[:maxlen]
            fn = '%s/%s' % (path, newfilename)
        return fn

    objects = AttachmentManager()

    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = versioning_models.VersionedGenericForeignKey('content_type', 'object_id')
    attachment_file = models.FileField(verbose_name=_('attachment'), upload_to=attachment_upload, storage=fs)

    class Meta:
        verbose_name = _('Attachment')
        verbose_name_plural = _('Attachments')
        ordering = ['-date_created']

    def __unicode__(self):
        return u'%s' % self.attachment_file.name

    @property
    def filename(self):
        return os.path.split(self.attachment_file.name)[1]


    # urls

    def get_absolute_url(self):
        if self.content_object:
            return self.content_object.get_absolute_url()
        return u''

    @models.permalink
    def get_delete_url(self):
        return ('attachments_delete', (self.v_id,))


    # permissions

    @classmethod
    def may_add(cls, user, parent=None):
        return parent.may_change(user)

    def may_change(self, user):
        if self.version:
            return False
        if self.content_object:
            return self.content_object.may_change(user)
        return False

    def may_delete(self, user):
        return self.may_change(user)

    @classmethod
    def get_report_list_for_object_body(cls, user, object, language=u'en', choices={}, level=1):
        attachments = cls.objects.attachments_for_object(object)
        if attachments:
            context = {'attachments': attachments, 'level': level}
            html = utils_reports.make_pdf_body(context, language=language, template='attachments/report.html')
            return [html,]
        return []

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        return ["links/report.css",]


def make_attachments_version(sender, instance, objects, **kwargs):
    if instance.published:
        filterkwarg={'published_dirty': True, 'version__isnull': True}
    else:
        filterkwarg={'dirty': True, 'version__isnull': True}
    if objects:
        for typelist in objects:
            if typelist:
                object_type = ContentType.objects.get_for_model(typelist[0])
                for item in Attachment.objects.filter(**filterkwarg).filter(content_type__pk=object_type.id,
                           object_id__in=[item.v_id for item in typelist]):
                    item.make_version(instance)
    else:
        for item in Attachment.objects.filter(**filterkwarg):
            item.make_version(instance)

versioning_models.post_version.connect(make_attachments_version, sender=versioning_models.Version)

def restore_attachments_version(sender, instance, objects, **kwargs):
    if objects:
        for typelist in objects:
            if typelist:
                object_type = ContentType.objects.get_for_model(typelist[0])
                for item in Attachment.objects.filter(content_type__pk=object_type.id,
                           object_id__in=[item.v_id for item in typelist]):
                    item.restore_version(instance)
    else:
        for item in Attachment.objects.get_newerobjects(instance):
            item.restore_version(instance)

versioning_models.restore_version.connect(restore_attachments_version, sender=versioning_models.Version)
