import datetime
from django.conf import settings
from django.contrib import messages
from django.core.paginator import Paginator, InvalidPage
from django.db.models import Q
from django import forms
from django.http import HttpResponse, HttpResponseRedirect, Http404, QueryDict
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext, loader
from django.utils.datastructures import MultiValueDict
from django.utils.dateformat import format
from django.utils.encoding import force_unicode
from django.utils.text import capfirst
from django.utils.translation import ugettext as _
from django.utils import translation
from django.template import Variable
from django.template.defaultfilters import escapejs
from main.context_processors.templates_context_processors import popup_base_template

from utils import urlhelpers, permissions, lists
from utils import reports as utils_reports

class RefererForm(forms.Form):
    referer = forms.CharField(required=False, widget=forms.HiddenInput)

class KeepFilterForm(forms.Form):
    filter = forms.CharField(required=False, widget=forms.HiddenInput)

class SendMailForm(forms.Form):
    sendmail = forms.BooleanField(required=False, label=_('Send via email'))

class SortForm(forms.Form):
    order_by = forms.CharField(required=False, widget=forms.HiddenInput)

class Inline(object):
    title = ''
    formset_class = None

    def get_formset(self, request, parent, **kwargs):
        """
        Return the formset instance

        """
        self.request = request
        self.parent = parent
        if request.method == 'POST':
            formset = self.get_formset_class()(data=request.POST, files=request.FILES, instance=parent, **kwargs)
        else:
            formset = self.get_formset_class()(instance=parent, **kwargs)
        formset.title = self.get_title()
        self.setup_formset(formset, parent)
        return formset

    def get_formset_class(self):
        return self.formset_class

    def setup_formset(self, formset, parent):
        pass

    def get_title(self):
        return self.title

    def save_instance(self, object):
        """
        Save and return model instance.

        """
        object.save()
        return object

    def save_instances(self, objectlist):
        """
        Save and return model instance.

        """
        res = []
        for object in objectlist:
            self.save_instance(object)
            res.append(object)
        return res

    def save_formset(self, formset):
        """
        Save formset, returning saved object.

        """
        return self.save_instances(formset.save(commit=False))

class ObjectView(object):
    """
    Generic view, parent for Add, Edit and Display views

    """
    request = None
    model = None

    form_class = None
    previewform_class = None
    inlines = []

    parent_class = None
    parent_field = None
    parent_tab = None
    parent = None

    objectpermission = None
    permission = None

    template = 'crud/edit.html'
    popup_template = None

    default_url = None
    currentmenuitem = ''
    message = None
    errormessages = []

    tabs = []
    tabviews = []
    currenttab = ''

    allow_change = True
    allow_delete = True

    allow_report = False

    def __new__(cls, *args, **kwargs):
        """
        Avoid thread-safety problems by returning a new view class for each request
        See http://codysoyland.com/2010/feb/3/thread-safe-object-oriented-views-django/
        """
        def view_wrapper(request, *args, **kwargs):
            view = object.__new__(cls)
            return view(request, *args, **kwargs)
        return view_wrapper

    def __call__(self, request, *args, **kwargs):
        self.request = request
        self.language = translation.get_language()
        self.previewform = None
        self.form = None
        self.refform = None
        self.formsets = []
        self.object = self.lookup_object(*args, **kwargs)
        self.setup_tabs()
        self.setup_tab_from_args(*args, **kwargs)
        self.setup_parent_from_args(*args, **kwargs)
        self.setup_from_args(*args, **kwargs)

        # permissions check
        c = self.check_permission(self.object)
        if c:
            return c

        result = self.process_request()
        if result:
            return result

        media = self.get_media()

        extra_context = {
                'object': self.object,
                'form': self.form,
                'preview': self.previewform,
                'preview_enabled': bool(self.previewform_class),
                'refform': self.refform,
                'formsets': self.formsets,
                'title': self.get_title(self.object),
                'currentmenuitem': self.get_currentmenuitem(self.object),
                'media': media,
                'errormessages': self.errormessages,
                'allow_report': self.allow_report
            }
        self.add_tabs_to_extra_context(extra_context, self.object)
        self.add_permissions_to_extra_context(extra_context, self.object)
        self.add_to_extra_context(extra_context, self.object)
        return self.get_response(extra_context)

    def lookup_object(self, *args, **kwargs):
        return self.model()

    def setup_tabs(self):
        pass

    def setup_parent_from_args(self, *args, **kwargs):
        if self.parent_class and self.object:
            self.parent = getattr(self.object, self.parent_field)

    def setup_tab_from_args(self, tab=None, *args, **kwargs):
        if self.tabs:
            if tab:
                if tab in [t['name'] for t in self.tabs]:
                    self.currenttab = tab
                else:
                    self.currenttab = self.tabs[0]['name']
            else:
                self.currenttab = self.tabs[0]['name']

    def setup_from_args(self, *args, **kwargs):
        pass

    def check_permission(self, object):
        """
        Default permission check

        """
        if self.permission:
            c = permissions.check_permission(self.request.user, self.permission)
            if c:
                return c
        if self.objectpermission:
            c = permissions.check_object_permission(self.request.user, object, self.objectpermission)
            if c:
                return c

    def process_request(self):
        self.errormessages = []
        if self.request.method == 'POST':
            all_valid = True
            preview = False
            self.refform = RefererForm(data=self.request.POST, prefix='management')
            if self.form_class:
                self.form = self.get_form(self.object)
                if self.form.is_valid():
                    if self.request.POST.get('preview','') != '' and self.previewform_class:
                        self.previewform = self.previewform_class(data=self.request.POST, files=self.request.FILES, instance=self.request.POST)
                        preview = True
                else:
                    all_valid=False
            if self.inlines:
                for inline in self.inlines:
                    formset = inline.get_formset(self.request, self.object)
                    formset.inline = inline
                    self.formsets.append(formset)
                    if not formset.is_valid():
                        all_valid=False
            if all_valid:
                if not preview:
                    if self.form:
                        self.object = self.save_form(self.form)
                    for formset in self.formsets:
                        formset.inline.save_formset(formset)
                    self.after_save()
                    self.set_message(self.object)
                    return self.get_redirect(self.refform, self.object)
            else:
                self.errormessages = [self.get_errormessage()]
        else:
            self.refform = RefererForm(prefix='management')
            self.refform.initial['referer'] = self.get_referer(self.object)
            if self.form_class:
                self.form = self.get_form(self.object)
            if self.inlines:
                for inline in self.inlines:
                    formset = inline.get_formset(self.request, self.object)
                    self.formsets.append(formset)

    def get_form(self, object, **formargs):
        """
        Default form instantiation

        """
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, files=self.request.FILES, instance=object, **formargs)
        else:
            form = self.form_class(instance=object, initial=self.get_initial(object), **formargs)
        self.setup_form(form, object)
        return form

    def setup_form(self, form, object):
        pass

    def get_initial(self, object):
        return {}

    def save_instance(self, object):
        """
        Save and return model instance.

        """
        object.save()
        return object

    def save_form(self, form):
        """
        Save form, returning saved object.

        """
        object = self.save_instance(form.save(commit=False))
        form.save_m2m()
        return object

    def after_save(self):
        pass

    def set_message(self, object):
        if self.message:
            messages.info(self.request, self.message)
##            self.request.user.message_set.create(message = self.message)

    def get_redirect(self, refform, object):
        """
        Return a HttpResponseRedirect based on refform
        argument and just-saved object.

        """

        if "_popup" in self.request.POST:
            return HttpResponse('<script type="text/javascript">opener.dismissPopup(window);</script>')

        if refform and refform.is_valid():
            r = refform.cleaned_data['referer']
            if r:
                return HttpResponseRedirect(r)
        return HttpResponseRedirect(self.get_default_url(object))

    def get_errormessage(self):
        return _('There were errors.')

    def get_referer(self, object):
        if self.request.META.get('HTTP_REFERER', None):
            return self.request.META['HTTP_REFERER']
        return self.get_default_url(object)

    def get_default_url(self, object):
        if self.default_url:
            return self.default_url
        else:
            if self.parent:
                return urlhelpers.absolute_url(self.parent, self.parent_tab)
        return '/'

    def get_title(self, object):
        return u'%s' % capfirst(self.model._meta.verbose_name)

    def get_currentmenuitem(self, object):
        return self.currentmenuitem


    def add_tabs_to_extra_context(self, extra_context, object):
        if self.tabs:
            tablist = []
            for t in self.tabs:
                include = False
                if t.get('permission'):
                    if self.request.user.has_perm(t['permission']):
                        include = True
                else:
                    include = True
                if include:
                    tl = t.copy()
                    if not tl.get('url'):
                        tl['url'] = self.get_tab_url(object, t['name'])
                    tablist.append(tl)
            extra_context['tablist'] = tablist
            extra_context['currenttab'] = self.currenttab

    def get_tab_url(self, object, tab):
        return urlhelpers.absolute_url(object, tab)

    def get_media(self):
        media = forms.widgets.Media()
        if self.form:
            media = media + self.form.media
        for fs in self.formsets:
            media = media + fs.media
        return media

    def add_permissions_to_extra_context(self, extra_context, object):
        pass

    def add_to_extra_context(self, extra_context, object):
        """
        Add items to extra_context.

        """
        pass

    def get_response(self, extra_context):
        """
        Return a HttpResponse object based on given request, template,
        and context.

        """
        if (self.request.method == 'POST' and "_popup" in self.request.POST) or self.request.GET.get('_popup'):
            popup_template = self.popup_template or self.template
            return render_to_response(popup_template, extra_context, RequestContext(self.request, processors=[popup_base_template]))
        return render_to_response(self.template, extra_context, RequestContext(self.request))


class AddObjectView(ObjectView):
    """
    Generic view to create instances of a model.

    """

    viewtype = 'add'
    objectpermission = 'add'

    def setup_parent_from_args(self, object_id=None, *args, **kwargs):
        if self.parent_class and object_id:
            self.parent = get_object_or_404(self.parent_class, pk=object_id)
            setattr(self.object, self.parent_field, self.parent)

    def check_permission(self, object):
        """
        Default permission check

        """
        if self.permission:
            c = permissions.check_permission(self.request.user, self.permission)
            if c:
                return c
        if self.objectpermission:
            permfunckwargs= self.parent and {'parent': self.parent} or {}
            c = permissions.check_object_permission(self.request.user, object, self.objectpermission, permfunckwargs=permfunckwargs)
            if c:
                return c

    def get_title(self, object):
        if self.parent:
            return _('New %(object_type)s for %(parent)s') % {'object_type': self.model._meta.verbose_name, 'parent': self.parent}
        return u'%s' % capfirst(self.model._meta.verbose_name)

    def save_form(self, form):
        object = form.save(commit=False)
        if self.parent_class:
            setattr(object, self.parent_field, self.parent)
        object = self.save_instance(object)
        form.save_m2m()
        return object

    def set_message(self, object):
        message = self.message
        if not message:
            message = u'%s: %s %s' % (capfirst(_('added')), object._meta.verbose_name, object,)
        messages.info(self.request, message)
##        self.request.user.message_set.create(message = force_unicode(message))

    def get_redirect(self, refform, object):
        """Special version, because returned js for popup is specific.
        """

        if "_popup" in self.request.POST:
            return HttpResponse('<script type="text/javascript">opener.dismissAddAnotherPopup(window, "%s", "%s");</script>' % \
            (object.pk, escapejs(object.__unicode__())))

        if refform and refform.is_valid():
            r = refform.cleaned_data['referer']
            if r:
                return HttpResponseRedirect(r)
        return HttpResponseRedirect(self.get_default_url(object))

class EditObjectView(ObjectView):
    """
    Generic view to update instances of a model.

    """
    viewtype = 'edit'
    objectpermission = 'change'

    def __call__(self, request, object_id=None, *args, **kwargs):
        self.object_id = object_id
        return super(EditObjectView, self).__call__(request, *args, **kwargs)

    def lookup_object(self,*args, **kwargs):
        return get_object_or_404(self.model, pk = self.object_id)

    def set_message(self, object):
        message = self.message
        if not message:
            message = u'%s: %s %s' % (capfirst(_('modified')), object._meta.verbose_name, object,)
        messages.info(self.request, message)
##        self.request.user.message_set.create(message = force_unicode(message))

    def get_title(self, object):
        return u'%s: %s' % (capfirst(self.model._meta.verbose_name), object)

class DisplayObjectView(EditObjectView):
    """
    Generic view to display an instance of a model.

    """
    viewtype = 'display'
    template = 'crud/display.html'
    objectpermission = None
    allow_report = True

    def get_form(self, object, **formargs):
        form = self.form_class(instance=object, **formargs)
        self.setup_form(form, object)
        return form

    def add_permissions_to_extra_context(self, extra_context, object):
        defaultbuttons = []
        if self.allow_change:
            if hasattr(object, 'may_change'):
                allowed = object.may_change(self.request.user)
            else:
                allowed = self.request.user.has_perm('%s.change_%s' % (object._meta.app_label.lower(), object._meta.object_name.lower()))
            if allowed:
                defaultbuttons.append({'url': urlhelpers.edit_url(object, self.currenttab), 'label': _('Change')})
        if self.allow_delete:
            if hasattr(object, 'may_delete'):
                allowed = object.may_delete(self.request.user)
            else:
                allowed = self.request.user.has_perm('%s.delete_%s' % (object._meta.app_label.lower(), object._meta.object_name.lower()))
            if allowed:
                defaultbuttons.append({'url': urlhelpers.delete_url(object), 'label': _('Delete')})
        extra_context['defaultbuttons'] = defaultbuttons

class DeleteObjectView(DisplayObjectView):
    """
    Generic view to delete an instance of a model.

    """
    viewtype = 'delete'
    template="crud/delete.html"
    question = None
    objectpermission = 'delete'

    def __call__(self, request, object_id=None, *args, **kwargs):
        self.object_id = object_id
        self.request = request
        form = None
        self.object = self.lookup_object(*args, **kwargs)
        self.setup_parent_from_args(*args, **kwargs)
        self.setup_from_args(*args, **kwargs)
        c = self.check_permission(self.object)
        if c:
            return c
        if request.method == 'POST':
            refform = RefererForm(data=request.POST, prefix='management')
            redir = self.get_redirect(refform, self.object)
            self.delete_instance(self.object)
            self.set_message(self.object)
            return redir
        else:
            refform = RefererForm(prefix='management')
            refform.initial['referer'] = self.get_referer(self.object)
            if self.form_class:
                form = self.get_form(self.object)
        extra_context = {
                'object': self.object,
                'form': form,
                'refform': refform,
                'deletequestion': self.get_question(self.object),
                'title': self.get_title(self.object),
                'currentmenuitem': self.get_currentmenuitem(self.object)
            }
        self.add_to_extra_context(extra_context, self.object)
        return self.get_response(extra_context)

    def delete_instance(self, object):
        object.delete()

    def set_message(self, object):
        message = self.message
        if not message:
            message = u'%s: %s %s' % (capfirst(_('deleted')), object._meta.verbose_name, object,)
        messages.info(self.request, message)
##        self.request.user.message_set.create(message = message)

    def get_question(self, object):
        question = self.question
        if not question:
            question = u'%s: %s %s' % (capfirst(_('delete')), object._meta.verbose_name, object,)
        return question

    def get_referer(self, object):
        ref = self.request.META.get('HTTP_REFERER', None)
        if ref:
            abs = urlhelpers.absolute_url(object)
            if abs:
                if not ref.find(abs):
                    return ref
                else:
                    return self.get_default_url(object)
        return self.get_default_url(object)

    def add_permissions_to_extra_context(self, extra_context, object):
        pass

class ReportObjectView(DisplayObjectView):
    """
    Generic view to create a report of an instance of a model.

    """
    form_class = None
    template = 'crud/report_select.html'
    choices = {}
    task = None

# TODO: add possibility for multiple selectforms (eg contributed by subitems)

    def process_request(self):
        self.render_pdf = True
        self.sendmail = False
        self.sendmailform = None
        if self.form_class:
            self.render_pdf = False
            self.errormessages = []
            if self.request.method == 'POST':
                self.refform = RefererForm(data=self.request.POST, prefix='management')
                self.form = self.get_form(self.object)
                self.sendmailform = SendMailForm(data=self.request.POST, prefix='sendmail')
                if self.form.is_valid() and self.sendmailform.is_valid():
                    self.render_pdf = True
                    self.choices = self.form.cleaned_data
                    self.sendmail = self.sendmailform.cleaned_data['sendmail']
                else:
                    self.errormessages = [self.get_errormessage()]
            else:
                self.refform = RefererForm(prefix='management')
                self.refform.initial['referer'] = self.get_referer(self.object)
                self.form = self.get_form(self.object)
                self.sendmailform = SendMailForm(prefix='sendmail')

    def get_form(self, object, **formargs):
        """ Do not pass instance into select form """
        if self.request.method == 'POST':
            form = self.get_form_class()(data=self.request.POST, files=self.request.FILES, **formargs)
        else:
            form = self.get_form_class()(initial=self.get_initial(object), **formargs)
        self.setup_form(form, object)
        return form

    def get_form_class(self):
        return self.form_class

    def add_to_extra_context(self, extra_context, object):
        if settings.USE_CELERY:
            extra_context['sendmailform'] = self.sendmailform

    def get_response(self, extra_context):
        if self.render_pdf:
            if self.sendmail:
                from main.tasks import mail_pdf_for_object
                try:
                    mail_pdf_for_object.delay(self.model, self.object.pk, self.request.user,self.language, self.choices)
                except:
                    raise Http404
                return render_to_response('crud/mail_confirmation.html', RequestContext(self.request))

            # for testing:
            if utils_reports.TESTPDF:
                return HttpResponse(self.get_pdf())

# test send_mail
#            reference = u'%s: %s' % (capfirst(self.model._meta.verbose_name), self.object)
#            utils_reports.send_mail(self.get_pdf(), self.request.user, reference=reference)
#
            return HttpResponse(self.get_pdf(), content_type='application/pdf')
        return render_to_response(self.template, extra_context, RequestContext(self.request))

    def get_pdf(self):
        return self.object.get_report(self.request.user, language=self.language, choices=self.choices)

class jEditObjectView(object):
    """
    Generic view to change one field in a form,
    eg via the jquery jeditable plugin
    on GET:
        returns the current value
    on POST:
        sets the value and returns the current value as presented in the sheet
    """
    request = None
    model = None
    permission = None
    objectpermission = 'change'
    form_class = None
    sheet_class = None

    def __new__(cls, *args, **kwargs):
        """
        Avoid thread-safety problems by returning a new view class for each request
        See http://codysoyland.com/2010/feb/3/thread-safe-object-oriented-views-django/
        """
        def view_wrapper(request, *args, **kwargs):
            view = object.__new__(cls)
            return view(request, *args, **kwargs)
        return view_wrapper

    def __call__(self, request, *args, **kwargs):
        self.request = request
        self.object = self.lookup_object(*args, **kwargs)
        self.check_permission(self.object)
        return self.process_request()

    def lookup_object(self, *args, **kwargs):
        if self.request.method == 'POST':
            id = self.request.POST.get('id')
        else:
            id = self.request.GET.get('id')
        if not id:
            raise Http404
        ids = id.split('_')
        self.field = '_'.join(ids[:-1])
        if self.field not in [f.name for f in self.model._meta.fields]:
            raise Http404
        self.object_id = ids[-1]
        return get_object_or_404(self.model, pk = self.object_id)

    def check_permission(self, object):
        """
        raises 404 if no permission
        """
        if self.permission:
            c = permissions.check_permission(self.request.user, self.permission, raise404=True)
            if c:
                return c
        if self.objectpermission:
            c = permissions.check_object_permission(self.request.user, object, self.objectpermission, raise404=True)
            if c:
                return c

    def process_request(self):
        if self.request.method == 'POST':
            value = self.request.POST.get('value')
            if not value:
                raise Http404
            data = {'id': [self.object_id]}
            data[self.field]= [value]
            data = MultiValueDict(data)
            form = self.form_class(instance=self.object, data=data)
            if form.is_valid():
                self.object = form.save()
            else:
                raise Http404
            # now return the html as displayed in the sheet
            if self.sheet_class:
                sheet = self.sheet_class(instance=self.object)
                result = sheet.fields[self.field].get_field(instance=self.object)
            else:
                result = getattr(self.object, self.field)
        else:
            # return the raw field data
            result = getattr(self.object, self.field)
        return HttpResponse(result, content_type='application/txt')


class TabContainer(object):
    tabviews = {}
    default = ''

    def __new__(cls, *args, **kwargs):
        """
        Avoid thread-safety problems by returning a new view class for each request
        See http://codysoyland.com/2010/feb/3/thread-safe-object-oriented-views-django/
        """
        def view_wrapper(request, *args, **kwargs):
            view = object.__new__(cls)
            return view(request, *args, **kwargs)
        return view_wrapper

    def __call__(self, request, tab=None, *args, **kwargs):
        tabviews = self.get_tabviews(request)
        t = tab or self.default
        view = tabviews.get(t)
        if not view:
            return permissions.redirect_to_not_allowed()
        return view(request, tab=t, *args, **kwargs)


    def get_tabviews(self, request):
        return self.tabviews

class Filter(object):
    form = None
    prefix = 'filter'
    filters = []
    default = {}

    def __call__(self, request, queryset, **kwargs):
        filterform = None
        qs = queryset
        params = {}
        filtered = False
        form = self.get_form()
        if form:
            if request.method == 'POST':
                if request.POST.get('submit_clear'):
                    filterform = self.call_form(form, prefix=self.prefix, **kwargs)
                else:
                    filterform = self.call_form(form, data=request.POST, prefix=self.prefix, **kwargs)
                    if filterform.is_valid():
                        Qobjs, params = lists.params_from_dict(self.filters, filterform.cleaned_data)
                        qs, filtered = self.apply_filter(request.user, queryset, Qobjs, params, filterform.cleaned_data)
                        if Qobjs or params:
                            filtered=True
                    else:
                        qs = queryset.none()
            else:
                Qobjs, params = lists.params_from_dict(self.filters,self.default)
                # GET request may contain filter info
                # use this to fill params
                Qobjsget, paramsget = lists.params_from_dict(self.filters, request.GET, prefix=self.prefix)
                Qobjs.extend(Qobjsget)
                params.update(paramsget)
                qs, filtered = self.apply_filter(request.user, queryset, Qobjs, params, request.GET)
                formdata = self.default.copy()
                formdata.update(lists.values_from_get(request.GET, prefix=self.prefix))
                filterform = self.call_form(form, initial=formdata, prefix=self.prefix, **kwargs)
            self.setup_form(filterform, request)
        return qs, filterform, filtered

    def apply_filter(self, user, queryset, Qobjs, params, cleaned_data):
        qs, filtered = self.extra_filter(queryset, cleaned_data)
        qs = qs.filter(*Qobjs, **params).distinct()
        return qs, filtered

    def extra_filter(self, queryset, data):
        """ Hook for custom filtering"""
        return queryset, False

    def get_form(self):
        return self.form

    def call_form(self, form, *args, **kwargs):
        return form(*args, **kwargs)

    def setup_form(self, form, request):
        pass

class ListView(object):
    """
    Generic list view
    Context:
        object_list
            list of objects
        paginator
            a ``QuerySetPaginator``object if pagination is enabled, None otherwise
        page_obj
            a ``Page`` object if pagination is enabled, None otherwise
    """
    request = None
    template = 'crud/list.html'
    paginate_by = None
    allow_empty = True
    title = ''
    currentmenuitem = ''
    queryset = None
    filter = None
    permission = None
    redirect_on_unique = False
    allow_pdf = False
    pdfselectform_class = None
    pdfsendmail = False
    listobject = None

    def __new__(cls, *args, **kwargs):
        """
        Avoid thread-safety problems by returning a new view class for each request
        See http://codysoyland.com/2010/feb/3/thread-safe-object-oriented-views-django/
        """
        def view_wrapper(request, *args, **kwargs):
            view = object.__new__(cls)
            return view(request, *args, **kwargs)
        return view_wrapper

    def __call__(self, request, *args, **kwargs):
        self.request = request

        self.language = translation.get_language()

        self.setup_from_args(*args, **kwargs)

        queryset = self.get_queryset(request)

        # permissions check
        c = self.check_permission()
        if c:
            return c

        # dispatch to pdf or csv
        if request.method == 'POST':
            if request.POST.get('submit_pdf'):
                return self.pdf(queryset)
        else:
            if request.GET.get('submit_pdf'):
                return self.pdf(queryset)

        filter_form = None
        filtered = False
        if self.filter:
            queryset, filter_form, filtered = self.filter(request, queryset)
            if filtered and self.redirect_on_unique:
                count = queryset.count()
                if count == 1:
                    return HttpResponseRedirect(self.url_for_unique(queryset[0]))

        if self.paginate_by:
            paginator = Paginator(queryset, self.paginate_by,
                                          allow_empty_first_page=self.allow_empty)
            page_number = self.get_page_number(request)
            try:
                page_obj = paginator.page(page_number)
            except InvalidPage:
                raise Http404
            object_list = page_obj.object_list
        else:
            object_list = queryset
            paginator = None
            page_obj = None
            if not self.allow_empty and len(queryset) == 0:
                raise Http404

        media = forms.widgets.Media()
        if filter_form:
            media = media + filter_form.media

        extra_context = {
            'object_list': object_list,
            'paginator': paginator,
            'page_obj': page_obj,
            'title': self.get_title(),
            'currentmenuitem': self.currentmenuitem,
            'filterform': filter_form,
            'filtered': filtered,
            'media': media,
            'allow_pdf': self.allow_pdf,
        }
        self.add_to_extra_context(extra_context)
        return self.get_response(extra_context)

    def setup_from_args(self, *args, **kwargs):
        pass

    def check_permission(self):
        """
        Default permission check

        """
        if self.permission:
            return permissions.check_permission(self.request.user, self.permission)

    def url_for_unique(self, object):
        return object.get_absolute_url()

    def get_page_number(self, request):
        if request.method == 'POST':
            page = request.POST.get('page',1)
        else:
            page = request.GET.get('page', 1)
        if not page:
            page = 1
        try:
            page_number = int(page)
        except ValueError:
            raise Http404
        return page_number

    def get_queryset(self, request):
        """
        Hook to provide a custom queryset based on the request.
        """
        return self.queryset._clone()

    def add_to_extra_context(self, extra_context):
        """
        Add items to extra_context.

        """
        pass

    def get_response(self, extra_context):
        return render_to_response(self.template, extra_context, RequestContext(self.request))

    def get_title(self):
        return self.title

    def pdf(self, queryset):
        self.sendmail = False
        self.filterdata = {}
        self.choices = {}
        self.order_by = None
        if self.pdfselectform_class or self.pdfsendmail:
            self.errormessages = []
            render_form = False
            self.sendmailform = None
            self.form = None
            if self.request.method == 'POST':
                self.filterform = KeepFilterForm(data=self.request.POST, prefix='filter')
                self.sortform= SortForm(data=self.request.POST, prefix='sort')
                if self.pdfselectform_class:
                    self.form = self.get_pdfselectform()
                self.sendmailform = SendMailForm(data=self.request.POST, prefix='sendmail')
                if (not self.form or self.form.is_valid()) and self.filterform.is_valid() and self.sendmailform.is_valid() and self.sortform.is_valid():
                    self.choices = self.form and self.form.cleaned_data or {}
                    self.sendmail = self.sendmailform.cleaned_data['sendmail']
                    if self.filter and self.filter.filters:
##                        Qobjs, params = lists.params_from_dict(self.filter.filters, QueryDict(self.filterform.cleaned_data['filter']), prefix=self.filter.prefix)
                        f = self.filter.get_form()
                        ff = f(data=QueryDict(self.filterform.cleaned_data['filter']), prefix=self.filter.prefix)
                        if ff.is_valid():
                            self.filterdata = ff.cleaned_data
                    self.order_by = self.sortform.cleaned_data.get('order_by')
                else:
                    self.errormessages = [_('There were errors.')]
                    render_form = True
            else:
                self.filterform = KeepFilterForm(prefix='filter')
                self.filterform.initial['filter'] = self.request.GET.urlencode()
                self.sortform= SortForm(data=self.request.GET, prefix='sort')
                if self.pdfselectform_class:
                    self.form = self.get_pdfselectform()
                if self.pdfsendmail:
                    self.sendmailform = SendMailForm(prefix='sendmail')
                render_form = True
            if render_form:
                extra_context = {
                        'title': _('Report'),
                        'form': self.form,
                        'filterform': self.filterform,
                        'sortform': self.sortform,
                        'errormessages': self.errormessages,
                    }
                if settings.USE_CELERY:
                    extra_context['sendmailform'] = self.sendmailform
                return render_to_response('crud/report_select.html', extra_context, RequestContext(self.request))
        else:
            if self.filter and self.filter.filters:
                f = self.filter.get_form()
                ff = f(data=self.request.POST, prefix=self.filter.prefix)
                if ff.is_valid():
                    self.filterdata = ff.cleaned_data
            self.sortform = SortForm(data=self.request.POST, prefix='sort')
            if self.sortform.is_valid():
                self.order_by = self.sortform.cleaned_data.get('order_by')

        # for testing:
        if utils_reports.TESTPDF:
            return HttpResponse(self.get_pdf())

        if self.sendmail:
            self.set_task()
            return render_to_response('crud/mail_confirmation.html', RequestContext(self.request))
# test send_mail
#        utils_reports.send_mail(self.get_pdf(), self.request.user, reference=self.listobject.get_title())
#
        return HttpResponse(self.get_pdf(), content_type='application/pdf')

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST)
        else:
            form = self.get_pdfselectform_class()()
        return form

    def get_pdfselectform_class(self):
        return self.pdfselectform_class

    def set_task(self):
        pass

    def get_pdf(self):
        if self.listobject:
            return self.listobject.get_report(self.request.user, self.filter, self.filterdata, self.language, self.choices, self.order_by)
        return u''

class ListObject(object):
    pdftemplate = 'crud/report.html'
    title = ''
    defaultsort = None

    def get_queryset(self):
        return self.queryset._clone()

    def extra_filter(self, queryset, data):
        """ Hook for custom filtering"""
        return queryset

    def add_to_extra_context(self, extra_context):
        pass

    def get_report(self, user, filter, filterdata, language, choices, order_by):
        Qobjs, params = lists.params_from_dict(filter.filters, filterdata)
        queryset = self.get_queryset()
        queryset = queryset.order_by(order_by or self.defaultsort)
        queryset, filtered = filter.apply_filter(user, queryset, Qobjs, params, filterdata)
        html = self.get_html(queryset, user, language, choices, order_by)
        html_strings = [html,]
        self.add_html(html_strings, queryset, user, language, choices)
        css_files = []
        self.add_css(css_files, queryset, user, language, choices)
        css_files = set(css_files)

        return utils_reports.make_pdf(html_strings, css_files, language=language)

    def get_html(self, queryset, user, language, choices, order_by):
        extra_context = {
            'object_list': queryset,
            'title': self.get_title(),
        }
        self.add_to_extra_context(extra_context)
        return utils_reports.make_pdf_body(extra_context, language=language, template=self.pdftemplate)

    def get_title(self):
        return self.title

    def add_css(self, css_files, queryset, user, language, choices):
        pass

    def add_html(self, html_strings, queryset, user, language, choices):
        pass

class TableListObject(ListObject):
    table_class = None
    table_classes = []

    def get_table(self, queryset, order_by):
        if self.table_class:
            table = self.table_class(data=queryset, order_by = order_by or self.defaultsort)
            table.classes = self.table_classes
            return table

        return None

    def get_html(self, queryset, user, language, choices, order_by):
        extra_context = {
            'table': self.get_table(queryset, order_by),
            'title': self.get_title(),
            'level': 1,
        }
        self.add_to_extra_context(extra_context)
        return utils_reports.make_pdf_body(extra_context, language=language, template=self.pdftemplate)


class TableListView(ListView):
    """
    using table from django-tables
    extra: csv from filtered list
    """
    table_class = None
    defaultsort = None
    csv_table_class = None
    allow_csv = False
    template = "crud/tablelist.html"
    csv_filename = "list.csv"

    def __call__(self, request, *args, **kwargs):
        self.request = request

        self.language = translation.get_language()

        self.setup_from_args(*args, **kwargs)

        queryset = self.get_queryset(request)

        # permissions check
        c = self.check_permission()
        if c:
            return c

        # dispatch
        if request.method == 'POST':
            if request.POST.get('submit_pdf'):
                return self.pdf(queryset)
        else:
            if request.GET.get('submit_pdf'):
                return self.pdf(queryset)

        filter_form = None
        filtered = False
        if self.filter:
            queryset, filter_form, filtered = self.filter(request, queryset)

        if request.method == 'POST':
            if request.POST.get('submit_csv'):
                return self.csv(queryset)
        else:
            if request.GET.get('submit_csv'):
                return self.csv(queryset)

        if self.filter:
            if filtered and self.redirect_on_unique:
                count = queryset.count()
                if count == 1:
                    return HttpResponseRedirect(self.url_for_unique(queryset[0]))

        sort_form, order_by = self.sort(request)

        table = self.get_table(request, queryset, order_by)

        if self.paginate_by:
            page_number = self.get_page_number(request)
            table.paginate(Paginator, self.paginate_by, page=page_number,
                allow_empty_first_page=self.allow_empty)

        media = forms.widgets.Media()
        if filter_form:
            media = media + filter_form.media

        extra_context = {
            'table': table,
            'title': self.get_title(),
            'currentmenuitem': self.currentmenuitem,
            'filterform': filter_form,
            'sortform': sort_form,
            'filtered': filtered,
            'media': media,
            'allow_csv': self.allow_csv,
            'allow_pdf': self.allow_pdf,
        }
        self.add_to_extra_context(extra_context)
        return self.get_response(extra_context)

    def sort(self, request):
        sort_form = None
        order_by = ''
        if request.method == 'POST':
            sort_form = SortForm(data=request.POST, prefix='sort')
            if sort_form.is_valid():
                order_by = sort_form.cleaned_data.get('order_by')
            else:
                order_by = ''
        else:
            order_by = request.GET.get('sort', self.defaultsort)
            sort_form = SortForm(prefix='sort', initial={'order_by': order_by})
        return sort_form, order_by

    def get_table(self, request, queryset, order_by):
        if self.table_class:
            return self.table_class(data=queryset, order_by=order_by)
        return None

    def csv(self, queryset):
        table = self.get_csv_table(self.request, queryset)
        if table:
            return table.as_csv(self.get_csv_filename())
        return None

    def get_csv_filename(self):
        return self.csv_filename

    def get_csv_table(self, request, queryset):
        if self.csv_table_class:
            return self.csv_table_class(queryset)
        return None

class DateBasedView(object):

    def __new__(cls, *args, **kwargs):
        """
        Avoid thread-safety problems by returning a new view class for each request
        See http://codysoyland.com/2010/feb/3/thread-safe-object-oriented-views-django/
        """
        def view_wrapper(request, *args, **kwargs):
            view = object.__new__(cls)
            return view(request, *args, **kwargs)
        return view_wrapper

    def __call__(self, request, year=None, month=None, *args, **kwargs):
        now = datetime.datetime.now()
        self.year = year and int(year) or now.year
        self.month = month and int(month) or 0
        if self.month < 0:
            self.month = 0
        if self.month > 12:
            self.month = 12
        if self.month == 0:
            start_date = datetime.date(self.year,1,1)
            end_date = datetime.date(self.year+1,1,1)
        else:
            start_date = datetime.date(self.year,self.month,1)
            if self.month > 11:
                end_date = datetime.date(self.year+1,1,1)
            else:
                end_date = datetime.date(self.year, self.month+1,1)
        return self._respond(request, start_date, end_date, *args, **kwargs)

    def _respond(self, request, start_date, end_date):
        # override this
        return HttpResponse('Not implemented')

    def _get_extra_context(self, ec, *args, **kwargs):
        ec['year'] = self.year
        ec['month'] = self.month
        if self.month:
            ec['period'] = format(datetime.date(self.year, self.month, 1), 'F Y')
        else:
            ec['period'] = self.year
        ec['links'] = self._get_sidebar(*args, **kwargs)


    def _get_sidebar(self, *args, **kwargs):
        links = []
        links.append(self._get_link(self.year - 1, 0 ,*args, **kwargs))
        link = self._get_link(self.year, 0 ,*args, **kwargs)
        if self.month == 0:
            del link['url']
        links.append(link)
        for i in range(1, 13):
            link = self._get_link(self.year, i,*args, **kwargs)
            if i == self.month:
                del link['url']
            links.append(link)
        links.append(self._get_link(self.year + 1, 0,*args, **kwargs))
        return links

    def _get_link(self, year, month, *args, **kwargs):
        # override this
        return {'url': '', 'link': ''}

class NoObjectView(ObjectView):
    """
    Generic view for views without object

    """
    template = 'crud/display.html'
    title = ''

    def __new__(cls, *args, **kwargs):
        """
        Avoid thread-safety problems by returning a new view class for each request
        See http://codysoyland.com/2010/feb/3/thread-safe-object-oriented-views-django/
        """
        def view_wrapper(request, *args, **kwargs):
            view = object.__new__(cls)
            return view(request, *args, **kwargs)
        return view_wrapper

    def __call__(self, request, *args, **kwargs):
        self.request = request
        self.language = translation.get_language()
        self.setup_tabs(*args, **kwargs)
        self.setup_from_args(*args, **kwargs)

        # permissions check
        c = self.check_permission()
        if c:
            return c

        extra_context = {
                'title': self.get_title(),
                'currentmenuitem': self.currentmenuitem,
            }
        self.add_tabs_to_extra_context(extra_context, *args, **kwargs)
        self.add_to_extra_context(extra_context)
        return self.get_response(extra_context)

    def get_title(self):
        return self.title

    def add_to_extra_context(self, extra_context):
        pass

    def setup_tabs(self, tab=None, *args, **kwargs):
        if self.tabs:
            if tab:
                if tab in [t['name'] for t in self.tabs]:
                    self.currenttab = tab
                else:
                    self.currenttab = self.tabs[0]['name']
            else:
                self.currenttab = self.tabs[0]['name']

    def setup_from_args(self, *args, **kwargs):
        pass

    def check_permission(self):
        if self.permission:
            return permissions.check_permission(self.request.user, self.permission)

    def add_tabs_to_extra_context(self, extra_context, *args, **kwargs):
        if self.tabs:
            tablist = []
            for t in self.tabs:
                tl = t.copy()
                tl['url'] = self.get_tab_url(t['name'])
                tablist.append(tl)
            extra_context['tablist'] = tablist
            extra_context['currenttab'] = self.currenttab

    def get_tab_url(self, tab):
        return u''

class ReportNoObjectView(NoObjectView):
    """
    Generic view to create a report

    """
    form_class = None
    pdfsendmail = False
    title = 'Report'

    def __call__(self, request, *args, **kwargs):
        self.request = request
        self.language = translation.get_language()
        self.setup_from_args(*args, **kwargs)

        # permissions check
        c = self.check_permission()
        if c:
            return c

        self.sendmail = False
        if self.form_class or self.pdfsendmail:
            self.errormessages = []
            render_form = False
            self.sendmailform = None
            self.form = None
            if self.request.method == 'POST':
                if self.form_class:
                    self.form = self.get_form()
                self.sendmailform = SendMailForm(data=self.request.POST, prefix='sendmail')
                if (not self.form or self.form.is_valid()) and self.sendmailform.is_valid():
                    self.choices = self.form and self.form.cleaned_data or {}
                    self.sendmail = self.sendmailform.cleaned_data['sendmail']
                else:
                    self.errormessages = [_('There were errors.')]
                    render_form = True
            else:
                if self.form_class:
                    self.form = self.get_form()
                if self.pdfsendmail:
                    self.sendmailform = SendMailForm(prefix='sendmail')
                render_form = True
            if render_form:
                extra_context = {
                        'form': self.form,
                        'title': self.get_title(),
                        'errormessages': self.errormessages,
                    }
                if settings.USE_CELERY:
                    extra_context['sendmailform'] = self.sendmailform
                return render_to_response('crud/report_select.html', extra_context, RequestContext(self.request))

        # for testing:
        if utils_reports.TESTPDF:
            return HttpResponse(self.get_pdf())

        if self.sendmail:
            self.set_task()
            return render_to_response('crud/mail_confirmation.html', RequestContext(self.request))
        return HttpResponse(self.get_pdf(), content_type='application/pdf')

    def set_task(self):
        pass

    def get_pdf(self):
        return u''

    def get_form(self, **formargs):
        """ Do not pass instance into select form """
        if self.request.method == 'POST':
            form = self.get_form_class()(data=self.request.POST, **formargs)
        else:
            form = self.get_form_class()(**formargs)
        return form

    def get_form_class(self):
        return self.form_class
