"""
Tests for exchange app
"""

from django import test
from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType

from utils.exchange import exchange
from utils import tests as utils_tests
from utils.versioning.versioning_test import models as vt_models

class ExchangeTestCase(test.TestCase):
    fixtures = ['dummyusers',]

    def test_exchange_simple(self):
        """
        Test dumping and loading objects
        """
        user = User.objects.get(pk=1)

        self.assertEquals(vt_models.Building.objects.count(), 0)
        self.assertEquals(vt_models.Appartment.objects.count(), 0)
        self.assertEquals(vt_models.CleaningLady.objects.count(), 0)
        self.assertEquals(vt_models.Appartment_CleaningLady.objects.count(), 0)

        c1 = vt_models.CleaningLady(name='c1')
        c1.save(user)

        c2 = vt_models.CleaningLady(name='c2')
        c2.save(user)

        b1 = vt_models.Building(name='b1')
        b1.save(user)

        b2 = vt_models.Building(name='b2')
        b2.save(user)

        a1 = vt_models.Appartment(name='a1', building=b1)
        a1.save(user)

        a1_c1 = vt_models.Appartment_CleaningLady(appartment=a1, cleaninglady=c1)
        a1_c1.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 2)
        self.assertEquals(vt_models.Appartment.objects.count(), 1)
        self.assertEquals(vt_models.CleaningLady.objects.count(), 2)
        self.assertEquals(vt_models.Appartment_CleaningLady.objects.count(), 1)

        # dump everything for b1
        objects = [b1]
        apps = b1.appartment_set.all()
        objects.extend(apps)
        ac = vt_models.Appartment_CleaningLady.objects.filter(appartment__in = apps)
        objects.extend(ac)
        cl = vt_models.CleaningLady.objects.filter(pk__in = [item.cleaninglady.id for item in ac])
        objects.extend(cl)

        data = exchange.dump(objects)

        # load everything as new objects
        exchange.load(user, data)

        self.assertEquals(vt_models.Building.objects.count(), 3)
        self.assertEquals(vt_models.Appartment.objects.count(), 2)
        self.assertEquals(vt_models.CleaningLady.objects.count(), 3)
        self.assertEquals(vt_models.Appartment_CleaningLady.objects.count(), 2)

        b3 = vt_models.Building.objects.all().order_by('-id')[0]
        self.assertEquals(b3.appartment_set.count(), 1)
        a2 = b3.appartment_set.all()[0]
        self.assertEquals(a2.cleaningladies.count(), 1)


        # test for load with external translation
        # dump everything for b1, but without b1
        objects = []
        apps = b1.appartment_set.all()
        objects.extend(apps)
        ac = vt_models.Appartment_CleaningLady.objects.filter(appartment__in = apps)
        objects.extend(ac)
        cl = vt_models.CleaningLady.objects.filter(pk__in = [item.cleaninglady.id for item in ac])
        objects.extend(cl)

        data = exchange.dump(objects)

        # load everything as new objects
        # translate references to b1 to b2
        ct = ContentType.objects.get_for_model(vt_models.Building).pk
        v_idmap = {(ct, b1.v_id): b2}
        exchange.load(user, data, v_idmap=v_idmap)

        self.assertEquals(vt_models.Building.objects.count(), 3)
        self.assertEquals(vt_models.Appartment.objects.count(), 3)
        self.assertEquals(vt_models.CleaningLady.objects.count(), 4)
        self.assertEquals(vt_models.Appartment_CleaningLady.objects.count(), 3)

        self.assertEquals(b2.appartment_set.count(), 1)
        a3 = b2.appartment_set.all()[0]
        self.assertEquals(a3.cleaningladies.count(), 1)

    def test_exchange_tree(self):
        """
        Test dumping and loading objects in tree
        """
        user = User.objects.get(pk=1)

        self.assertEquals(vt_models.DiskTreeElement.objects.count(), 0)
        self.assertEquals(vt_models.Folder.objects.count(), 0)

        # create a tree element
        folder = vt_models.Folder(name=u'Base')
        treeelement = folder.save(user, parent=None, treeelementkwargs={})
        # and another
        folder = vt_models.Folder(name=u'Child 1')
        folder.save(user, parent=treeelement, treeelementkwargs={})
        # and another
        folder = vt_models.Folder(name=u'Child 2')
        folder.save(user, parent=treeelement, treeelementkwargs={})

        self.assertEquals(vt_models.DiskTreeElement.objects.count(), 3)
        self.assertEquals(vt_models.Folder.objects.count(), 3)

        # dump everything for b1
        objects = []
        items = vt_models.DiskTreeElement.objects.filter(elementtype=vt_models.FOLDER_ID
                ).values_list('object_id', flat=True)
        if items:
            objects.extend(list(vt_models.Folder.objects.filter(v_id__in=items)))

        objects.extend(list(vt_models.DiskTreeElement.tree.all()))

        data = exchange.dump(objects)

        # load everything as new objects
        exchange.load(user, data)

        self.assertEquals(vt_models.DiskTreeElement.objects.count(), 6)
        self.assertEquals(vt_models.Folder.objects.count(), 6)
