import datetime
from django.template import loader
from utils.urlhelpers import edit_url, delete_url
from django.utils.encoding import smart_unicode, smart_str
from django.utils.html import escape

##__all__ = (
##    'Column', 'TextColumn', 'NumberColumn', 'DateColumn', 'TemplatedColumn',
##    'DeleteColumn',
##)

class Column(object):
    """Represents a single column of a table.

    ``verbose_name`` defines a display name for this column used for output.

    ``name`` is the internal name of the column. Normally you don't need to
    specify this, as the attribute that you make the column available under
    is used. However, in certain circumstances it can be useful to override
    this default, e.g. when using ModelTables if you want a column to not
    use the model field name.

    ``default`` is the default value for this column. If the data source
    does provide ``None`` for a row, the default will be used instead. Note
    that whether this effects ordering might depend on the table type (model
    or normal). Also, you can specify a callable, which will be passed a
    ``BoundRow`` instance and is expected to return the default to be used.

    Additionally, you may specify ``data``. It works very much like
    ``default``, except it's effect does not depend on the actual cell
    value. When given a function, it will always be called with a row object,
    expected to return the cell value. If given a string, that name will be
    used to read the data from the source (instead of the column's name).

    Note the interaction with ``default``. If ``default`` is specified as
    well, it will be used whenver ``data`` yields in a None value.

    You can use ``visible`` to flag the column as hidden by default.
    However, this can be overridden by the ``visibility`` argument to the
    table constructor. If you want to make the column completely unavailable
    to the user, set ``inaccessible`` to True.

    Setting ``sortable`` to False will result in this column being unusable
    in ordering. You can further change the *default* sort direction to
    descending using ``direction``. Note that this option changes the actual
    direction only indirectly. Normal und reverse order, the terms
    django-tables exposes, now simply mean different things.
    """

    ASC = 1
    DESC = 2

    # Tracks each time a Column instance is created. Used to retain order.
    creation_counter = 0

    def __init__(self, verbose_name=None, name=None, default=None, data=None,
                 visible=True, inaccessible=False, sortable=False,
                 direction=ASC, link=None, column_classes=None, link_permission=None):
        self.verbose_name = verbose_name
        self.name = name
        self.default = default
        self.data = data
        self.visible = visible
        self.inaccessible = inaccessible
        self.sortable = sortable
        self.direction = direction
        self.link = link
        self.column_classes = column_classes
        self.link_permission = link_permission

        self.creation_counter = Column.creation_counter
        Column.creation_counter += 1

    def _set_direction(self, value):
        if isinstance(value, basestring):
            if value in ('asc', 'desc'):
                self._direction = (value == 'asc') and Column.ASC or Column.DESC
            else:
                raise ValueError('Invalid direction value: %s' % value)
        else:
            self._direction = value

    direction = property(lambda s: s._direction, _set_direction)

    def as_html(self, value, boundrow, context):
        if not value:
            return u''
        value = self.format_value(value)
        if self.link:
            if self.link_permission:
                user = context.get('user')
                allowed = False
                object = boundrow.data
                if user:
                    if hasattr(object, 'may_'+self.link_permission):
                        allowed = getattr(object, 'may_'+self.link_permission)(user)
                    else:
                        allowed = user.has_perm('%s.%s_%s' % (object._meta.app_label.lower(), self.link_permission, object._meta.object_name.lower()))
            else:
                allowed = True
            if allowed: # permission ok
                if callable(self.link):
                    url = self.link(boundrow)
                else:
                    url = self.link
                if url:
                    return u'<a href="%s">%s</a>' % (url, smart_unicode(value))
        return smart_unicode(value)

    def format_value(self, value):
        return escape(value)

    def as_csv(self, value):
        if not value:
            return u''
        if isinstance(value, basestring):
            value = self.format_value(value).replace('\r\n','\n')
        value = smart_unicode(value)
        return smart_str(value, encoding='iso-8859-1', errors='replace')

class TextColumn(Column):
    pass

class NumberColumn(Column):
    pass

class DateColumn(Column):
    def __init__(self, format=None, *args, **kwargs):
        self.format = format
        super(DateColumn, self).__init__(*args, **kwargs)

    def format_value(self, value):
        if self.format:
            return u'%s' % datetime.date.strftime(value, self.format)
        return value

class TemplatedColumn(Column):
    def __init__(self, template='', *args, **kwargs):
        self.template = template
        super(TemplatedColumn, self).__init__(*args, **kwargs)

    def as_html(self, value, boundrow, context):
        if self.template:
            t = loader.get_template(self.template)
            context.push()
            context['object'] = boundrow.data
            output = t.render(context)
            context.pop()
            return output
        return ''

class PermissionedTemplatedColumn(TemplatedColumn):
    def __init__(self, permission, template, *args, **kwargs):
        self.permission = permission
        kwargs['data'] = lambda x: ''
        super(PermissionedTemplatedColumn, self).__init__(template=template, *args, **kwargs)

    def as_html(self, value, boundrow, context):
        user = context.get('user')
        allowed = False
        object = boundrow.data
        if user:
            if hasattr(object, 'may_'+self.permission):
                allowed = getattr(object, 'may_'+self.permission)(user)
            else:
                allowed = user.has_perm('%s.%s_%s' % (object._meta.app_label.lower(), self.permission, object._meta.object_name.lower()))
        if allowed: # permission ok
            self.add_to_context(context, object)
            c = context
            c.update( {
                    'object' : boundrow.data,
                })
            return super(PermissionedTemplatedColumn, self).as_html(value, boundrow, context)
        return ''

    def add_to_context(self, context, object):
        pass

class DeleteColumn(PermissionedTemplatedColumn):
    def __init__(self, verbose_name=' ', *args, **kwargs):
        super(DeleteColumn, self).__init__(permission='delete', template='tables/delete.html',
            verbose_name=verbose_name, *args, **kwargs)

    def add_to_context(self, context, object):
        context['delete_url'] = delete_url(object)

class EditColumn(PermissionedTemplatedColumn):
    def __init__(self, verbose_name=' ', *args, **kwargs):
        super(EditColumn, self).__init__(permission='edit', template='columns/edit.html',
        verbose_name=verbose_name, *args, **kwargs)

    def add_to_context(self, context, object):
        context['edit_url'] = edit_url(object)

class MultipleColumn(Column):
    def __init__(self, format=None, joiner = u', ', *args, **kwargs):
        self.format = format
        self.joiner = joiner
        super(MultipleColumn, self).__init__(*args, **kwargs)

    def as_html(self, value, boundrow, context):
        list = []
        for relobject in value:
            if self.format:
                v = self.format(relobject)
            else:
                v = relobject
            if self.link:
                if callable(self.link):
                    url = self.link(relobject)
                else:
                    url = self.link
                list.append(u'<a href="%s">%s</a>' % (url, v))
            else:
                list.append(u'%s' % v)
        return smart_unicode(self.joiner.join(list))

    def as_csv(self, value):
        list = []
        for relobject in value:
            list.append(smart_unicode(relobject))
        return smart_str(self.joiner.join(list), encoding='iso-8859-1', errors='replace')
