"""
base forms for versioned models

There is a lot of un-DRY here because our model methods save and delete need
user info.

"""

from django import forms
from django.db import models
from utils import forms as utils_forms
from utils import sheets
from utils.versioning import models as versioning_models
from django.utils.text import capfirst
from django.forms.formsets import DELETION_FIELD_NAME

def removefields(form):
    "extracted from form and sheet to stay DRY"
    if form.fields.get('dirty'):
        del form.fields['dirty']
    if form.fields.get('published'):
        del form.fields['published']
    if form.fields.get('published_dirty'):
        del form.fields['published_dirty']
    if form.fields.get('new'):
        del form.fields['new']
    if form.fields.get('deleted'):
        del form.fields['deleted']
    if form.fields.get('v_id'):
        del form.fields['v_id']
    if form.fields.get('user_created'):
        del form.fields['user_created']
    if form.fields.get('user_changed'):
        del form.fields['user_changed']
    if form.fields.get('date_created'):
        del form.fields['date_created']
    if form.fields.get('date_changed'):
        del form.fields['date_changed']
    if form.fields.get('version'):
        del form.fields['version']

class VersionedForeignKeyField(sheets.ForeignKeyField):
    def get_compare_data(self, instance):
        foreign_object = self.get_data(instance)
        if foreign_object:
            return getattr(foreign_object, 'v_id')
        return u''

class VersionedForm(utils_forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(VersionedForm, self).__init__(*args, **kwargs)
        removefields(self)

def Versionedformfield_callback(field, **kwargs):
    label = capfirst(field.verbose_name)
    if isinstance(field, versioning_models.VersionedForeignKey):
        return VersionedForeignKeyField(data=field.name, name=field.name, label=label, **kwargs)
    return sheets.formfield_callback(field, **kwargs)

class VersionedSheet(sheets.Sheet):
    formfield_callback = Versionedformfield_callback
    def __init__(self, *args, **kwargs):
        super(VersionedSheet, self).__init__(*args, **kwargs)
        removefields(self)

class VersionedInlineFormSetMetaclass(type):
    def __new__(cls, name, bases, attrs):
        try:
            parents = [b for b in bases if issubclass(b, VersionedInlineFormSet)]
        except NameError:
            # We are defining InlineFormSet itself.
            parents = None
        new_class = super(VersionedInlineFormSetMetaclass, cls).__new__(cls, name, bases,
                            attrs)
        if not parents:
            return new_class

        if not hasattr(new_class, 'can_order') and 'can_order' not in attrs:
            new_class.can_order = False
        if not hasattr(new_class, 'can_delete') and 'can_delete' not in attrs:
            new_class.can_delete = False
        if not hasattr(new_class, 'max_num') and 'max_num' not in attrs:
            new_class.max_num = None
        if not hasattr(new_class, 'extra') and 'extra' not in attrs:
            new_class.extra = 3
        if not hasattr(new_class, 'absolute_max') and 'absolute_max' not in attrs:
            new_class.absolute_max = 1000
        if not hasattr(new_class, 'validate_max') and 'validate_max' not in attrs:
            new_class.validate_max = False
        if hasattr(new_class, 'model') and hasattr(new_class, 'parent'):
            if new_class.model and new_class.parent:
                fk = (i for i in new_class.model._meta.fields if i.name == new_class.parent).next()
                if not isinstance(fk, versioning_models.VersionedForeignKey):
                    raise Exception(u'Error in definition of InlineFormset subclass %s. The parent attribute is not a VersionedForeignKey' % name)
                new_class.fk = fk

        return new_class

class VersionedInlineFormSet(forms.models.BaseModelFormSet):
    __metaclass__ = VersionedInlineFormSetMetaclass

    def __init__(self, data=None, files=None, instance=None,
                 save_as_new=False, prefix=None, queryset=None):
        if instance is None:
            self.instance = self.fk.to()
        else:
            self.instance = instance
        self.save_as_new = save_as_new
        if queryset is None:
            queryset = self.model._default_manager
        qs = queryset.filter(**{self.parent: self.instance}).get_versionedobjects(version=versioning_models.WorkVersion)
        super(VersionedInlineFormSet, self).__init__(data, files, prefix=prefix,
                                                queryset=qs)

    def initial_form_count(self):
        if self.save_as_new:
            return 0
        return super(VersionedInlineFormSet, self).initial_form_count()

    def total_form_count(self):
        if self.save_as_new:
            return super(VersionedInlineFormSet, self).initial_form_count()
        return super(VersionedInlineFormSet, self).total_form_count()

    def _construct_form(self, i, **kwargs):
        form = super(VersionedInlineFormSet, self)._construct_form(i, **kwargs)
        if self.save_as_new:
            form.data[form.add_prefix(self._pk_field.name)] = None
            form.data[form.add_prefix(self.fk.name)] = None

        # use v_id here
        setattr(form.instance, self.fk.get_attname(), self.instance.v_id)
        return form

    #@classmethod
    def get_default_prefix(cls):
        accessor_name = cls.fk.related_name or (cls.model._meta.object_name.lower() + '_set')
        return accessor_name
    get_default_prefix = classmethod(get_default_prefix)

    def add_fields(self, form, index):
        super(VersionedInlineFormSet, self).add_fields(form, index)
        if self._pk_field == self.fk:
            name = self._pk_field.name
            kwargs = {'pk_field': True}
        else:
            # The foreign key field might not be on the form, so we poke at the
            # Model field to get the label, since we need that for error messages.
            name = self.fk.name
            kwargs = {
                'label': getattr(form.fields.get(name), 'label', capfirst(self.fk.verbose_name))
            }
        kwargs['to_field'] = 'v_id'
        form.fields[name] = forms.models.InlineForeignKeyField(self.instance, **kwargs)

        # Add the generated field to form._meta.fields if it's defined to make
        # sure validation isn't skipped on that field.
        if form._meta.fields:
            if isinstance(form._meta.fields, tuple):
                form._meta.fields = list(form._meta.fields)
            form._meta.fields.append(self.fk.name)

    def get_unique_error_message(self, unique_check):
        unique_check = [field for field in unique_check if field != self.fk.name]
        return super(VersionedInlineFormSet, self).get_unique_error_message(unique_check)


# The next functions were overridden because we need the user in the objects delete method

    def save(self, user, commit=True):
        """Saves model instances for every form, adding and changing instances
        as necessary, and returns the list of instances.
        """
        if not commit:
            self.saved_forms = []
            def save_m2m():
                for form in self.saved_forms:
                    form.save_m2m()
            self.save_m2m = save_m2m
        return self.save_existing_objects(user, commit=commit) + self.save_new_objects(user, commit=commit)

    def save_existing_objects(self, user, commit=True):
        self.changed_objects = []
        self.deleted_objects = []
        if not self.get_queryset():
            return []

        saved_instances = []
        for form in self.initial_forms:
            pk_name = self._pk_field.name
            raw_pk_value = form._raw_value(pk_name)

            # clean() for different types of PK fields can sometimes return
            # the model instance, and sometimes the PK. Handle either.
            pk_value = form.fields[pk_name].clean(raw_pk_value)
            pk_value = getattr(pk_value, 'pk', pk_value)

            obj = self._existing_object(pk_value)
            if self.can_delete:
                raw_delete_value = form._raw_value(DELETION_FIELD_NAME)
                should_delete = form.fields[DELETION_FIELD_NAME].clean(raw_delete_value)
                if should_delete:
##                    self.deleted_objects.append(obj)
                    obj.delete(user)
                    continue
            if form.has_changed():
                self.changed_objects.append((obj, form.changed_data))
                saved_instances.append(self.save_existing(user, form, obj, commit=commit))
                if not commit:
                    self.saved_forms.append(form)
        return saved_instances

    def save_new_objects(self, user, commit=True):
        self.new_objects = []
        for form in self.extra_forms:
            if not form.has_changed():
                continue
            # If someone has marked an add form for deletion, don't save the
            # object.
            if self.can_delete:
                raw_delete_value = form._raw_value(DELETION_FIELD_NAME)
                should_delete = form.fields[DELETION_FIELD_NAME].clean(raw_delete_value)
                if should_delete:
                    continue
            self.new_objects.append(self.save_new(user, form, commit=commit))
            if not commit:
                self.saved_forms.append(form)
        return self.new_objects

    def save_new(self, user, form, commit=True):
        # Use commit=False so we can assign the parent key afterwards, then
        # save the object.
        obj = form.save(commit=False)
        setattr(obj, self.fk.name, self.instance)
        if commit:
            obj.save(user)
        # form.save_m2m() can be called via the formset later on if commit=False
        if commit and hasattr(form, 'save_m2m'):
            form.save_m2m()
        return obj

    def save_existing(self, user, form, instance, commit=True):
        """Saves and returns an existing model instance for the given form."""
        obj = form.save(commit=False)
        obj.save(user)
        return obj

