# version views

import datetime

from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.db.models.loading import get_model, get_models
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.dateformat import format
from django.utils.translation import ugettext_lazy as _

from utils import crud, permissions
from utils import tables
from utils.versioning import models as app_models
from utils.versioning import forms as app_forms
from utils.versioning import tables as app_tables
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version

def versioning_navigationlist(request):
    nl = [
            {'url': reverse('versioning_versions'),
                'label': _('Versions')}
            ]
    if request.user.has_perm('versioning.display_versions'):
        nl.extend([
            {'url': reverse('versioning_changedobjects'),
                'label': _('Changed')},
            {'url': reverse('versioning_changedpublicobjects'),
                'label': _('Changed publ.')},
        ])
    return nl

class MixIn(object):
    "Common items for object views"
    model = app_models.Version
    allow_change=False
    allow_delete=False

    tabs = [
        {'name': 'version',
            'label': _('Version')},
        {'name': 'items',
            'label': _('Items')},
    ]

    def setup_tabs(self):
        if not self.request.user.has_perm('versioning.display_versions'):
            self.tabs = []

class display_version(MixIn, crud.DisplayObjectView):
    objectpermission = 'display'
    template = "versioning/version.html"

    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def add_to_extra_context(self, extra_context, object):
        crud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        extra_context['navigationlist'] =  versioning_navigationlist(self.request)
        extra_context['may_activate'] = object.may_activate(self.request.user)
        extra_context['may_restore'] = object.may_restore(self.request.user)
        extra_context['may_merge'] = object.may_merge(self.request.user)
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion

class display_items(MixIn, crud.DisplayObjectView):
    objectpermission = 'display'

    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def add_to_extra_context(self, extra_context, object):
        crud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        extra_context['navigationlist'] =  versioning_navigationlist(self.request)
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion
        if object.may_display_collected_items(self.request.user):
            table = app_tables.ItemsTable(object.collect_items(self.request.user))
            extra_context['table'] = table

class display(MixIn, crud.TabContainer):
    tabviews = {
        'version': display_version(),
        'items': display_items(),
    }
    default = 'version'

class versions(crud.TableListView):
    "List of versions"

    table_class = app_tables.VersionsTable
    defaultsort = '-date_versioned'
    queryset = app_models.Version.objects.all()
    paginate_by = 100
    title = _('Versions')

    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def get_queryset(self, request):
        return app_models.Version.get_allowed_display_objects(request.user)

    def add_to_extra_context(self, extra_context):
        super(versions, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] = versioning_navigationlist(self.request)
        if app_models.Version.may_add(self.request.user):
            extra_context['tablebuttons'] = [
                    {'url': reverse('versioning_version_add'),
                     'label': _('New version'),},
                 ]
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion

class add(MixIn, crud.AddObjectView):
    form_class = app_forms.VersionForm
    tabs = []

    def save_instance(self, object):
        object.save(self.request.user, propagate=False)
        return object

    def get_initial(self, object):
        return {'name': _('Status from %s') % format(datetime.datetime.now(), settings.DATETIME_FORMAT)}

    def get_redirect(self, refform, object):
        return HttpResponseRedirect(reverse('versioning_version_create_version', args=[object.id,]))

    def set_message(self, object):
        pass

class create_version(MixIn, crud.DisplayObjectView):
    allow_change=False
    allow_delete=False
    template = "versioning/create_version.html"

    def get_title(self, object):
        return _('Version')

    def setup_from_args(self, *args, **kwargs):
        # get the active version
        (self.version, self.specialversion) = get_current_version(self.request)

    def add_to_extra_context(self, extra_context, object):
        super(create_version, self).add_to_extra_context(extra_context, object)
        extra_context['navigationlist'] = versioning_navigationlist(self.request)
        extra_context['version'] = self.version
        extra_context['specialversion'] = self.specialversion

def create_version_ajax(request, object_id):
    v = get_object_or_404(app_models.Version, pk=object_id)
    if not app_models.Version.may_add(request.user):
        return permissions.redirect_to_not_allowed()
    app_models.post_version.send(sender=v.__class__, instance=v, objects=[])
    return HttpResponse("OK", content_type='text/plain')

def activate(request, object_id=None):
    v = get_object_or_404(app_models.Version, pk=object_id)
    if not v.may_activate(request.user):
        return permissions.redirect_to_not_allowed()
    request.session['version_id'] = v.id
    return HttpResponseRedirect(v.get_absolute_url())

def deactivate(request):
    try:
        del request.session['version_id']
    except KeyError:
        pass
    if request.META.get('HTTP_REFERER', None):
        return HttpResponseRedirect(request.META['HTTP_REFERER'])
    return HttpResponseRedirect('/')

class restore(MixIn, crud.DisplayObjectView):
    objectpermission = 'restore'
    template = 'versioning/restore.html'
    tabs = []

    def add_to_extra_context(self, extra_context, object):
        extra_context['helptext'] = _('''Restoring a version sets the workversion back to the state of that version. If you have unversioned changes in your workversion, they will be saved in a version first.''')

    def process_request(self):
        self.restore = False
        if self.request.method == 'POST':
            self.restore = True

    def get_response(self, extra_context):
        if self.restore:
            self.object.restore(self.request.user)
            self.set_message(self.object)
            return HttpResponseRedirect(self.object.get_absolute_url())
        return render_to_response(self.template, extra_context, RequestContext(self.request))

    def set_message(self, object):
        message = u"Restored version '%s'" % object.name
        messages.info(self.request, message)

class merge(MixIn, crud.DisplayObjectView):
    objectpermission = 'merge'
    template = 'versioning/merge.html'
    tabs = []

    def add_to_extra_context(self, extra_context, object):
        extra_context['previous_version'] = object.previous_version()
        extra_context['helptext'] = _('''Merging a version with the previous version will incorporate the changes from the previous version into this version. The previous version will be removed.''')

    def process_request(self):
        self.merge = False
        if self.request.method == 'POST':
            self.merge = True

    def get_response(self, extra_context):
        if self.merge:
            self.object.merge_previous(self.request.user)
            self.set_message(self.object)
            return HttpResponseRedirect(self.object.get_absolute_url())
        return render_to_response(self.template, extra_context, RequestContext(self.request))

    def set_message(self, object):
        message = u"Merged version '%s'" % object.name
        messages.info(self.request, message)

class history(versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    template = 'versioning/history.html'
    def __call__(self, request, app_label, model_name, *args, **kwargs):
        self.model = get_model(app_label, model_name)
        return super(history, self).__call__(request, *args, **kwargs)

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        if self.request.user.has_perm('versioning.display_versions'):
            extra_context['history'] = object.history()
        else:
            extra_context['history'] = object.history_public()
        extra_context['buttons'] = [
            {'url': object.get_absolute_url(),
                'label': _('Back')},
        ]

def history_url_for_obj(obj):
    return reverse('versioning_history', kwargs={
                        'app_label': obj._meta.app_label,
                        'model_name': obj._meta.model_name,
                        'object_id': obj.v_id
                    })

class changedobjects(crud.TableListView):
    "List of objects changed since last version"

    table_class = app_tables.ChangedObjectsTable
    paginate_by = 100
    title = _('Changed objects')
    filterkwarg = {'dirty': True}
    permission = 'versioning.display_versions'

    def get_queryset(self, request):
        if self.paginate_by:
            page_number = self.get_page_number(request)
            maxitems = self.paginate_by * page_number
        objects= []
        count = 0
        for cl in get_models():
            if issubclass(cl, app_models.VersionedModel):
                if count <= maxitems:
                    obj = cl.objects.filter(version__isnull=True, **self.filterkwarg)
                    for item in obj:
                        objects.append({
                            'itemtype': item._meta.verbose_name,
                                'item': item,
                        })
                        count += 1
                else:
                    c = cl.objects.filter(version__isnull=True, **self.filterkwarg).count()
                    for i in range(c):
                        objects.append({})
        return objects

    def add_to_extra_context(self, extra_context):
        super(changedobjects, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] = versioning_navigationlist(self.request)

class changedpublicobjects(changedobjects):
    "List of objects changed compared to last published version"
    title = _('Objects changed since latest published version')
    filterkwarg = {'published_dirty': True}
